// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Mobius Forensic Toolkit
// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
// Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "api_metadata.h"
#include "api_dataholder.h"

// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//! \brief create PyList from metadata
//! \param metadata metadata object
//! \return list
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
PyObject *PyList_FromMetadata (const mobius::metadata& metadata)
{
  // create list
  PyObject *ret = PyList_New (0);
  if (!ret)
    return nullptr;

  // populate list
  for (auto attr : metadata)
    {
      // create attr object
      api_dataholder_o *pyobj = api_dataholder_new ();
      if (!pyobj)
        {
          Py_CLEAR (ret);
          return nullptr;
        }

      api_dataholder_setattr (pyobj, "name", attr.name);
      api_dataholder_setattr (pyobj, "description", attr.description);
      api_dataholder_setattr (pyobj, "datatype", attr.datatype);
      api_dataholder_setattr (pyobj, "value", attr.value);

      PyList_Append (ret, (PyObject *) pyobj);
      Py_DECREF (pyobj);
    }

  return ret;
}
