// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Mobius Forensic Toolkit
// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
// Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//! \brief  C++ API module wrapper
//! \author Eduardo Aguiar
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <Python.h>
#include "imagefile.h"

// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//! \brief function get_imagefile_types
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
PyObject*
func_get_imagefile_types (PyObject *, PyObject *)
{
  PyObject *ret = PyList_New (0);

  if (!ret)
    return nullptr;

  try
    {
      for (const auto& i : mobius::imagefile::get_imagefile_types ())
        {
          PyObject *t = PyTuple_New (4);
          if (!t)
            {
              Py_CLEAR (ret);
              return nullptr;
            }

          PyTuple_SetItem (t, 0, PyString_FromString (i.type.c_str ()));
          PyTuple_SetItem (t, 1, PyString_FromString (i.description.c_str ()));
          PyTuple_SetItem (t, 2, PyString_FromString (i.extensions.c_str ()));
          PyTuple_SetItem (t, 3, PyBool_FromLong (i.is_writeable));
          PyList_Append (ret, t);
          Py_DECREF (t);
        }
    }
  catch (const std::runtime_error& e)
    {
      Py_DECREF (ret);
      PyErr_SetString (PyExc_Exception, e.what ());
      ret = nullptr;
    }

  return ret;
}
